cv::Mat
CuboidUtil::loadDepth( std::string depth_path )
{
    cv::Mat dep;
    if ( depth_path.find("raw") != std::string::npos )
    {
        am::util::cv::loadDepthAsBinary<ushort,ushort>( dep, depth_path, 1. );
    }
    else if ( depth_path.find("dat") != std::string::npos )
    {
        // Tianjia's NYU, read float, and convert it to ushort
        am::util::cv::loadDepthAsBinary<ushort,float>( dep, depth_path, 1000. );
    }
    else
    {
        dep = cv::imread( depth_path, cv::IMREAD_UNCHANGED );
    }

    return dep;
}

// hfirst
// Tout is the format of the data file, Timg is the format of cv::Mat
template <typename Timg, typename Tout>
int
loadDepthAsBinary( ::cv::Mat &img, std::string path, float alpha = 1.f )
{
    FILE *fp = fopen( path.c_str(), "rb" );
    if (!fp)
    {
        printf( "loadDepthAsBinary failed, when opening %s\n", path.c_str() );
        return false;
    }

    int w,h, read_size = 0;
    read_size += fread( &h,sizeof(int),1,fp );
    read_size += fread( &w,sizeof(int),1,fp );
    if ( read_size != 2 )
    {
        std::cerr << "[" << __func__ << "]: " << "w or h not read properly: " << w << ", " << h << std::endl;
    }

    Tout *p_depth = new Tout[ h * w ];

    read_size = fread( p_depth, sizeof(Tout), w*h, fp );
    if ( read_size != w*h )
        std::cerr << "[" << __func__ << "]: " << "read size != " << w*h << std::endl;

    img.create( h, w, CV_16UC1 );
    int p_depth_index = 0;
    for ( unsigned y = 0; y < img.rows; ++y  )
    {
        for ( unsigned x = 0; x < img.cols; ++x, ++p_depth_index )
        {
            img.at<Timg>(y,x) = p_depth[ p_depth_index ] * alpha;
        }
    }

    fclose( fp );
    if ( p_depth )
        delete [] p_depth;

    return EXIT_SUCCESS;
}
